@@ -39,7 +39,7 @@ stepchk <- function(oldstep, cvec, deltac, limwrd, ind,
         }
 #  check whether lower limit has been reached twice in a row
         if (limflg) {
-          if (limwrd) ind <- 1 else limwrd <- TRUE
+          if (limwrd[1]) ind <- 1 else limwrd <- TRUE
         } else limwrd <- FALSE
   return(list(newstep, ind, limwrd))
 }
